(* 
 * Copyright 2012 Damien Guichard  
 *  
 * Licensed under the EUPL, Version 1.1 only (the "Licence");
 * You may not use this work except in compliance with the Licence. 
 * You may obtain a copy of the Licence at: 
 *  
 * http://ec.europa.eu/idabc/eupl5  
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the Licence for the specific language governing permissions
 * and limitations under the Licence. 
 *) 
 
module type Dictionary
=
sig
	type ('a,'b) t
	val empty : ('a,'b) t
	val add   : 'a -> 'b -> ('a,'b) t -> ('a,'b) t (* failwith "duplicate identifier" *)
	val find  : 'a -> ('a,'b) t -> 'b  (* raise Not_found *)
end

module type Taxonomy
=
sig
	type t
	type set
	val singleton : string -> set * t
	val from_string : string -> set -> t (* raise Not_found *)
	val to_string : t -> string
	val unique : t -> int
end 

module type TaxonomyTree
=
sig
	include Taxonomy
	val add_childs_to : string list -> t -> set -> unit (* failwith "identifier already in taxonomy" *)
end 

module type ER_DiGraph
=
sig
	type t
	type name
	type entity =
		| Entity  of name
		| EntityI of name * int
		| EntityR of name * int
		| EntityN of name * string
		| EntityS of name * string
		| EntityF of name * float
	type relation =
		name
	type edge =
		{
		mutable src: entity; src_mark: string;
		mutable rel: relation;
		mutable dst: entity; dst_mark: string;
		}
	type vertex =
		| Vertex
		| Source of edge
		| Destination of edge
	type pattern =
		(edge * vertex * vertex) list
	val empty : t
	val insert : edge list -> t -> t
	val select : (unit -> unit) -> pattern -> t -> unit
	val marshal_flags : Marshal.extern_flags list
end
